const isTextField = node => {
  return ['input', 'textarea'].indexOf(node.nodeName.toLowerCase()) > -1
}
const onFocusAdded = node => {
  if (!isTextField(node)) {
    return
  }
  node.parentNode.className += ' active'
}

const onFocusLose = node => {
  if (!node.parentNode || !isTextField(node)) { return }
  node.parentNode.classList.remove('active');
  node.parentNode.classList.remove('filled');

  if (node.value) {
    node.parentNode.classList.add('filled');
  }
}

const syncFields = root => {
  if (!root.querySelectorAll) { return }
  root.querySelectorAll('input,textarea').forEach(node => {
    onFocusLose(node)
  })
}

window.addEventListener('focus', e => onFocusAdded(document.activeElement), true)
window.addEventListener('blur', e => onFocusLose(e.target), true)
window.addEventListener('DOMNodeInserted', e => syncFields(e.target), false)
window.addEventListener('DOMContentLoaded', e => syncFields(document), false)

export default {
  syncFields,
  onFocusLose
}
